from __future__ import unicode_literals
from django.contrib.auth import get_user_model
from django import forms
from django.http.response import HttpResponse
from django.template.defaultfilters import yesno
from django.utils.html import escape
import six
from advanced_reports.backoffice.contrib.mixins import BackOfficeReportMixin

from advanced_reports.backoffice.shortcuts import action
from advanced_reports.defaults import AdvancedReport, ActionException, BootstrapReport
from oemfoe_todos_app.models import TodoList


User = get_user_model()


class UserForm(forms.ModelForm):
    class Meta:
        model = User
        fields = ('first_name', 'last_name')


class UserReport(BootstrapReport):
    models = (User,)
    fields = ('email', 'first_name', 'last_name',)
    help_text = 'This is a help text!'
    links = ((u'Refresh', '.'),)

    item_actions = (
        action(method='edit', verbose_name='Edit', form=UserForm, form_via_ajax=True, group='no_superuser'),
        action(method='send_reminder_email', verbose_name='Send reminder email', individual_display=True),
    )
    multiple_actions = True
    template = 'advanced_reports/bootstrap/report.html'
    item_template = 'advanced_reports/bootstrap/item.html'
    date_range = 'last_login'

    def queryset_request(self, request):
        return User.objects.all()

    def get_username_html(self, item):
        return u'<a href="#/user/%d/">%s</a>' % (item.pk, escape(item.username))

    def get_html_for_value(self, value):
        if isinstance(value, bool):
            return yesno(value, '<i class="glyphicon glyphicon-ok-circle">,<i class="glyphicon glyphicon-remove-circle">')
        return super(UserReport, self).get_html_for_value(value)

    def edit(self, item, form):
        form.save()

    def send_reminder_email(self, item):
        if item.pk % 2 == 0:
            raise ActionException('Could not deliver email')

    def verify_action_group(self, item, group):
        if not group: return True
        elif group == 'no_superuser' and not item.is_superuser: return True
        return False


class NoModel(object):
    def __init__(self, value):
        self.value = value

    @property
    def square(self):
        return self.value * self.value


class CSVForm(forms.Form):
    filename = forms.CharField(initial='numbers.csv', required=True)


class SumForm(forms.Form):
    factor = forms.IntegerField(initial=1, required=True)


class NoModelReport(BootstrapReport):
    verbose_name = 'number'
    verbose_name_plural = 'numbers'
    fields = ('value', 'square',)
    items_per_page = 10
    multiple_actions = True
    search_fields = ('value', 'square',)

    item_actions = (
        action(method='calculate_sum', verbose_name='Calculate sum', individual_display=False,
               form=SumForm, form_via_ajax=True),
        action(method='export_as_csv_view', verbose_name='Export selection as CSV',
               individual_display=True, form=CSVForm, form_via_ajax=True)
    )

    def queryset_request(self, request):
        return [NoModel(number) for number in xrange(1000)]

    def get_item_id(self, item):
        return item.value

    def get_item_for_id(self, item_id):
        return NoModel(int(item_id))

    def calculate_sum_multiple(self, items, form):
        factor = form.cleaned_data['factor']
        return {
            'dialog_content': 'The sum is: %d' % (sum((item.value for item in items)) * factor),
            'dialog_style': {'width': '300px'}
        }

    def export_as_csv_view(self, item, form):
        return self.export_as_csv_view_multiple([item], form)

    def export_as_csv_view_multiple(self, items, form):
        import csv
        filename = form.cleaned_data['filename']
        response = HttpResponse()
        writer = csv.writer(response, dialect='excel')
        writer.writerow(('Value', 'Square'))
        for item in items:
            writer.writerow((item.value, item.square))
        response['Content-Type'] = 'application/csv'
        response['Content-Disposition'] = 'attachment; filename="%s"' % filename
        return response


class NewStyleReport(BootstrapReport):
    model = User
    fields = 'first_name', 'last_name', 'email'

    @action('Say hello', css_class='btn-primary')
    def hello(self, item):
        return 'Hello, %s!' % escape(self.request.user.first_name)

    @action('Edit user', form=UserForm)
    def edit(self, item, form):
        form.save()

    @action('Enter factor', form=SumForm, is_report_action=True)
    def factor(self, form):
        if form.cleaned_data['factor'] == 2:
            raise ActionException("I don't like factor two :-(")
        return 'The given factor was: %d' % form.cleaned_data['factor']


class TodoListForm(forms.ModelForm):
    class Meta:
        model = TodoList
        fields = ('name', 'owner')


class TodoListReport(BackOfficeReportMixin, BootstrapReport):
    model = TodoList
    fields = ('name', 'owner')
    search_fields = ('name',)
    sortable_fields = fields
    multiple_actions = True

    @action('New Todo List', form=TodoListForm, is_report_action=True, css_class='btn-primary')
    def new(self, form):
        form.save()

    @action('Edit', form=TodoListForm, css_class='btn-warning')
    def edit(self, todo_list, form):
        form.save()

    @action('Delete', confirm='Are you sure you want to delete %(name)s?', css_class='btn-danger')
    def delete(self, todo_list):
        todo_list.delete()

    def get_name_decorator(self, todo_list):
        return self.link_to(todo_list)



from __future__ import unicode_literals
from django.core.urlresolvers import resolve, reverse


def get_backoffice_from_path(path):
    """
    Based on the given ``path``, return the ``BackOfficeBase`` subclass instance serving this path.

    :param path: The path to find the ``BackOfficeBase`` subclass instance for.
    :return: A ``BackOfficeBase`` subclass instance.
    """
    resolved = resolve(path)
    handle_path = reverse('%s:handle' % resolved.namespace, current_app=resolved.app_name)
    resolved_handle = resolve(handle_path)
    return resolved_handle.func.__self__


class BackOfficeReportMixin(object):
    """
    This mixin adds BackOffice-specific functionality to an ``AdvancedReport`` subclass.
    """

    def get_current_backoffice(self):
        """
        Gives the current ``BackOfficeBase`` subclass instance serving this report.

        :return: A ``BackOfficeBase`` subclass instance.
        """
        return get_backoffice_from_path(self.request.path)

    def link_to(self, instance):
        """
        Returns a HTML decorator providing, if available, a web link to the given model instance.

        :param instance: The model instance.
        :return: A function accepting a HTML string which returns a new HTML string.
        """
        backoffice = self.get_current_backoffice()
        bo_model = backoffice.get_model(model=instance.__class__)

        if bo_model is not None:
            return lambda h: '<a link-to="%s">%s</a>' % ("{model: '%s', id: %d}" % (bo_model.slug, instance.pk), h)

        elif hasattr(instance, 'get_absolute_url'):
            return lambda h: '<a href="%s">%s</a>' % (instance.get_absolute_url(), h)

        return lambda h: h



__version__ = '0.9.7'


REGISTRY = {}


def register(advreport):
    REGISTRY[advreport.slug] = advreport


def get_report_for_slug(slug):
    return REGISTRY.get(slug, lambda: None)()


def get_report_or_404(slug):
    advreport = get_report_for_slug(slug)
    if advreport is None:
        from django.http import Http404
        raise Http404('No AdvancedReport matches the given query.')

    advreport.internal_mode = False
    advreport.report_header_visible = True
    return advreport



#!/usr/bin/env python
from deployer.client import start
from deployer.host.ssh import SSHHost
from deployer.node import Node
import os
from deployer.utils.string_utils import esc1

REPO_DIRECTORY = os.environ.get('REPO_DIRECTORY', '/home/user/advreports')
BRANCH = os.environ.get('BRANCH', 'master')
REPOSITORY = 'https://github.com/vikingco/django-advanced-reports.git'
SSH_HOST = os.environ.get('SSH_HOST', 'vm.oemfoeland.com')
SSH_USER = os.environ.get('SSH_USER', 'user')
SSH_PASSWORD = os.environ.get('SSH_PASSWORD')
VIRTUALENV = os.environ.get('VIRTUALENV', '/home/user/env/advreports')
RUNIT_SERVICE = os.environ.get('RUNIT_SERVICE', 'advreports')

PROJECT_DIRECTORY = os.path.join(REPO_DIRECTORY, 'advreport_test_project')
PYTHON = os.path.join(VIRTUALENV, 'bin/python')
PIP = os.path.join(VIRTUALENV, 'bin/pip')


class oemfoeland_host(SSHHost):
    address = SSH_HOST
    username = SSH_USER
    password = SSH_PASSWORD


class AdvreportTestProjectDeployment(Node):
    class Hosts:
        host = oemfoeland_host

    def git_clone(self):
        self.hosts.run("git clone '%s' --branch='%s' '%s'" % (esc1(REPOSITORY), esc1(BRANCH), (REPO_DIRECTORY)))

    def git_fetch(self):
        with self.hosts.cd(REPO_DIRECTORY):
            self.hosts.run('git fetch origin')

    def git_checkout(self):
        with self.hosts.cd(REPO_DIRECTORY):
            self.hosts.run('git stash')
            self.hosts.run("git checkout '%s'" % (esc1(BRANCH),))
            self.hosts.run("git pull origin '%s'" % (esc1(BRANCH),))
            self.hosts.run('git stash pop')

    def pip_install(self):
        with self.hosts.cd(PROJECT_DIRECTORY):
            self.hosts.run("'%s' install -r requirements.txt" % esc1(PIP))

    def django_manage(self, command):
        with self.hosts.cd(PROJECT_DIRECTORY):
            self.hosts.run("'%s' manage.py %s" % (esc1(PYTHON), command))

    def django_collectstatic(self):
        self.django_manage('collectstatic --noinput')

    def django_migrate(self):
        self.django_manage('migrate')

    def restart_app(self):
        self.hosts.sudo("sv restart '%s'" % (esc1(RUNIT_SERVICE),))

    def deploy(self):
        self.git_fetch()
        self.git_checkout()
        self.pip_install()
        self.django_collectstatic()
        self.django_migrate()
        self.restart_app()


if __name__ == '__main__':
    start(AdvreportTestProjectDeployment)



from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.http.response import Http404, HttpResponse
from django.template.response import TemplateResponse
from django.test import TestCase, Client
from django.test.client import RequestFactory
import six
from advanced_reports.backoffice.api_utils import ViewRequestParameters, to_json
from advanced_reports.backoffice.base import BackOfficeBase, BackOfficeView

from advanced_reports.backoffice.examples.backoffice import test_backoffice
from advanced_reports.backoffice.examples.views import SimpleView


class BackOfficeBackend(TestCase):
    def setUp(self):
        user, created = User.objects.get_or_create(username='admin',
                                                   password='admin',
                                                   first_name='Admin',
                                                   last_name='User',
                                                   email='admin@example.com',
                                                   is_active=True,
                                                   is_staff=True,
                                                   is_superuser=True)
        user.set_password('admin')
        user.save()

        self.user2, created2 = User.objects.get_or_create(username='admin2',
                                                     password='admin',
                                                     first_name='Admin2',
                                                     last_name='User',
                                                     email='admin2@example.com',
                                                     is_active=True,
                                                     is_staff=True,
                                                     is_superuser=True)
        self.user2.set_password('admin')
        self.user2.save()

        self.home_url = reverse(test_backoffice.name + ':home', current_app=test_backoffice.app_name)
        self.client = Client()
        self.client.post(self.home_url, {'username': 'admin', 'password': 'admin'})

    def _reverse(self, name, args=None, kwargs=None):
        pass

    def test_default_context(self):
        """The default context contains usable values"""
        context = test_backoffice.default_context()
        self.assertEqual(context['backoffice'], test_backoffice)
        self.assertTrue('api_url' in context)
        self.assertTrue('root_url' in context)

    def test_home_view(self):
        response = self.client.get(self.home_url)
        self.assertEqual(response.status_code, 200)
        self.assertIn('Hi, <strong>Admin</strong>', response.content)

    def test_default_define_urls(self):
        self.assertEqual(BackOfficeBase().define_urls(), ())

    def test_login_as(self):
        self.client.get(self.home_url + 'login/as/' + six.text_type(self.user2.pk) + '/')
        response = self.client.get(self.home_url, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertIn('Hi, <strong>Admin2</strong>', response.content)
        response = self.client.get(self.home_url + 'end/login/as/', follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertIn('Hi, <strong>Admin</strong>', response.content)
        self.client.get(self.home_url + 'end/login/as/')

    def test_logout(self):
        response = self.client.get(self.home_url + 'logout/')
        self.assertEqual(response.status_code, 200)
        self.assertNotIn('Hi,', response.content)


class RequestWithViewParams(object):
    def __init__(self, params, user, post=False):
        if post:
            self.view_params = ViewRequestParameters(RequestFactory().post('/', params,
                                                                           content_type='application/json'))
        else:
            self.view_params = ViewRequestParameters(RequestFactory().get('/', params))
        self.user = user


class PermissionView(SimpleView):
    permission = 'can_eat_monsters'
    template = SimpleView.template

    def post(self, request):
        return self.get(request)


class PostView(SimpleView):
    template = SimpleView.template
    permission = 'some_permission'

    def get(self, request):
        return TemplateResponse(request, self.template, self.get_context(request)), {'extra': True}

    def post(self, request):
        return self.get(request)

    def do_something(self, request):
        return 'something'

    def myview(self, request):
        return HttpResponse('Hello!')


class BackOfficeViewTestCase(TestCase):
    @classmethod
    def setUpClass(cls):
        cls.user = User.objects.create_superuser('view_user', 'view-user@example.com', 'p')
        cls.user2 = User.objects.create_user('view_user2', 'view-user2@example.com', 'p')
        test_backoffice.register_view(SimpleView)
        test_backoffice.register_view(PermissionView)
        test_backoffice.register_view(PostView)

    def test_registration(self):
        v = test_backoffice.get_view('simple')
        self.assertIsNotNone(v)

    def test_get_view_content(self):
        r = test_backoffice.api_get_view(RequestWithViewParams({'view_slug': 'simple'}, self.user))
        self.assertEqual(r['slug'], 'simple')
        self.assertGreater(len(r['content']), 0)

        r = test_backoffice.api_get_view(RequestWithViewParams({'view_slug': 'post'}, self.user))
        self.assertEqual(r['extra'], True)

    def test_non_existing_view_slug(self):
        r = test_backoffice.api_get_view(RequestWithViewParams({'view_slug': 'nonexisting'}, self.user))
        self.assertIn('View with slug "nonexisting" does not exist.', r['content'])

    def test_get_permission(self):
        r = test_backoffice.api_get_view(RequestWithViewParams({'view_slug': 'permission'}, self.user2))
        self.assertEqual(r['content'], '')

    def test_post(self):
        with self.assertRaises(NotImplementedError):
            test_backoffice.api_post_view(RequestWithViewParams({'view_slug': 'simple'}, self.user))
        r = test_backoffice.api_post_view(RequestWithViewParams({'view_slug': 'post'}, self.user))
        self.assertEqual(r['slug'], 'post')

    def test_post_permission(self):
        r = test_backoffice.api_post_view(RequestWithViewParams({'view_slug': 'permission'}, self.user2))
        self.assertEqual(r.content, 'You are not allowed to post data to the view "permission".')

    def test_post_view_action(self):
        request_body = to_json({
            'method': 'do_something',
            'view_params': {'view_slug': 'post'},
            'params': {}
        })
        r = test_backoffice.api_post_view_action(RequestWithViewParams(request_body, self.user, post=True))
        self.assertEqual(r, 'something')

    def test_post_view_action_not_allowed(self):
        request_body = to_json({
            'method': 'do_something',
            'view_params': {'view_slug': 'post'},
            'params': {}
        })
        r = test_backoffice.api_post_view_action(RequestWithViewParams(request_body, self.user2, post=True))
        self.assertEqual(r.content, 'You are not allowed to post data to the view "post".')

    def test_post_view_action_non_existing_method(self):
        request_body = to_json({
            'method': 'non_existing_method',
            'view_params': {'view_slug': 'post'},
            'params': {}
        })
        with self.assertRaises(Http404):
             test_backoffice.api_post_view_action(RequestWithViewParams(request_body, self.user, post=True))

    def test_get_view_view(self):
        params = {'view_slug': 'post', 'method': 'myview'}
        r = test_backoffice.api_get_view_view(RequestWithViewParams(params, self.user))
        self.assertContains(r, 'Hello!')

    def test_get_view_view_not_allowed(self):
        params = {'view_slug': 'post', 'method': 'myview'}
        r = test_backoffice.api_get_view_view(RequestWithViewParams(params, self.user2))
        self.assertContains(r, 'You are not allowed to view data from the view "post".', status_code=403)

    def test_get_view_view_non_existing_method(self):
        params = {'view_slug': 'post', 'method': 'non_existing_method'}
        with self.assertRaises(Http404):
            test_backoffice.api_get_view_view(RequestWithViewParams(params, self.user))

    def test_post_view_view(self):
        params = {'view_slug': 'post', 'method': 'myview'}
        r = test_backoffice.api_post_view_view(RequestWithViewParams(params, self.user))
        self.assertContains(r, 'Hello!')

    def test_FOO(self):
        request_body = to_json({
            'method': 'FOO',
            'view_params': {'view_slug': 'post'},
            'params': {}
        })
        with self.assertRaises(NotImplementedError):
            test_backoffice.api_post_view_action(RequestWithViewParams(request_body, self.user, post=True))




from __future__ import unicode_literals


# Compatibility layer for pagination
try:
    import django_ajax
except ImportError:
    from django.core.paginator import Paginator
    def paginate(request, object_list, per_page):
        paginator = Paginator(object_list, per_page)
        return paginator.page(request.GET.get('page', 1))
else:
    from django_ajax.pagination import paginate as old_paginate
    def paginate(request, object_list, per_page):
        return old_paginate(request, object_list, per_page, use_get_parameters=True)



#!/usr/bin/env python
from setuptools import setup, find_packages
import advanced_reports

setup(
    name="django-advanced-reports",
    version=advanced_reports.__version__,
    url='https://github.com/vikingco/django-advanced-reports',
    license='BSD',
    description="Advanced reports for Django",
    long_description=open('README.rst', 'r').read(),
    author='Jef Geskens, VikingCo nv',
    packages=find_packages('.'),
    package_data = {'advanced_reports': [
                'static/*.js', 'static/*/*.js', 'static/*/*/*.js',
                'static/*.css', 'static/*/*.css', 'static/*/*/*.css',
                'static/*.png', 'static/*/*.png', 'static/*/*/*.png', 'static/*/*/*/*.png',
                'templates/*.html', 'templates/*/*.html', 'templates/*/*/*.html', 'templates/*/*/*/*.html',
                ],},
    zip_safe=False, # Don't create egg files, Django cannot find templates in egg files.
    include_package_data=True,
    classifiers=[
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Operating System :: OS Independent',
        'Environment :: Web Environment',
        'Framework :: Django',
    ],
    install_requires=[
        'Django',
        'six',
        'django-widget-tweaks',
        'pygments',
    ]
)



from collections import OrderedDict

from django import template
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext as _
from django.utils.text import capfirst

import re
import HTMLParser
import json as json_lib


register = template.Library()


@register.filter
def angularize_form(value, model_prefix=None):
    """
    Make a Django form ready for AngularJS

    :param value: a HTML form
    :param model_prefix: if specified, will be used as the container for the form data
    :return: a HTML form made ready for AngularJS
    """

    def replace_name_func(match):
        name = match.groups()[0]
        short_name = name.split(u'-', 1)[1] if u'-' in name else name
        model = short_name if model_prefix is None else u'%s.%s' % (model_prefix, short_name)
        return u' name="%(name)s" ng-model="%(model)s"' % locals()

    def replace_value_func(match):
        model, value = 1, 3
        groups = list(match.groups())
        parsed_value = HTMLParser.HTMLParser().unescape(groups[value])
        groups[model] = u'%s" ng-init="%s=\'%s\'' % (groups[model], groups[model], parsed_value.replace(u"'", u"\\'"))
        print repr(groups)
        return u''.join(groups)

    replaced_name = re.sub(r'\s+name="([^"]+)"', replace_name_func, value)
    replaced_value = re.sub(r'(<[^>]+\s+ng-model=")([^"]+)("[^>]+\s+value=")([^"]+)("[^>]*>)', replace_value_func, replaced_name)
    return mark_safe(replaced_value)


@register.filter
def json(value):
    """
    Convert a simple Python object to JSON

    :param value: a simple Python object
    :return: the JSON for the simple Python object
    """
    return json_lib.dumps(value)


@register.filter
def pretty_join(value_list, seperator=', '):
    '''
    Example:
    >>> pretty_join(['a', 'b', 'c'])
    u'a, b and c'
    '''
    output = seperator.join(capfirst(value) for value in value_list)

    # find last occurence
    index = output.rfind(seperator)
    if index >= 0:
        # replace it
        output = u'%s %s %s' % (output[0:index],
                                _(u'and'),
                                output[index-1 + len(seperator):])
    return output


@register.filter
def inline_errors(value):
    form = value
    err_list = OrderedDict({})

    # sort errors
    for key,errors in form.errors.items():
        for error in errors:
            if error not in err_list:
                err_list[error] = []
            if key in form.fields:
                label = form.fields[key].label
                if label:
                    label = _(label)
                    label = label.replace(u'?', u'')
                    label = label.replace(u'!', u'')
                    label = label.replace(u':', u'')
                    err_list[error].append(label)
                else:
                    err_list[error].append(key)
            else:
                err_list[error].append(key)

    # generate
    output = []
    for error, fields in err_list.items():
        if error == _('This field is required.'):
            if len(fields) == 1:
                output.append(_("%s is required.") % (pretty_join(fields)))
            else:
                output.append(_("%s are required.") % (pretty_join(fields)))
        else:
            output.append(error)

    return output



from __future__ import unicode_literals
from django.contrib.auth.models import User
from django.db import models
from django.utils.encoding import python_2_unicode_compatible


@python_2_unicode_compatible
class TodoList(models.Model):
    owner = models.ForeignKey(User)
    name = models.CharField(max_length=255)

    def __str__(self):
        return '{0} owned by {1}'.format(self.name, self.owner.get_full_name())


@python_2_unicode_compatible
class TodoItem(models.Model):
    todo_list = models.ForeignKey(TodoList)
    name = models.CharField(max_length=255)
    done = models.DateTimeField(blank=True, null=True)

    def __str__(self):
        return '{0} from {1}'.format(self.name, self.todo_list)



from importlib import import_module

from advanced_reports.backoffice.base import BackOfficeView
from django.template.loaders.app_directories import Loader
from pygments import highlight
from pygments.lexers.python import PythonLexer
from pygments.lexers.templates import HtmlDjangoLexer
from pygments.formatters.html import HtmlFormatter
import inspect


class ExampleIncludeTemplateView(BackOfficeView):
    template = 'advanced_reports/backoffice/views/exampleinclude.html'

    def get_extra_context(self, request):
        include = request.view_params.get('include')
        html, _ = Loader().load_template_source(include)
        html_snippet = highlight(html, HtmlDjangoLexer(), HtmlFormatter(full=False, noclasses=True))
        return {'html': html_snippet, 'path': include}


class ExampleIncludePythonView(BackOfficeView):
    template = 'advanced_reports/backoffice/views/exampleinclude.html'

    def get_extra_context(self, request):
        include = request.view_params.get('include')
        module, member = include.rsplit('.', 1)
        module = import_module(module)
        source = inspect.getsource(getattr(module, member))
        html_snippet = highlight(source, PythonLexer(), HtmlFormatter(full=False, noclasses=True))
        return {'html': html_snippet, 'path': include}



from __future__ import unicode_literals
from django.contrib import messages
from django.shortcuts import get_object_or_404
from advanced_reports.backoffice.base import BackOfficeModel, BackOfficeTab, BackOfficeView
from advanced_reports.backoffice.examples.backoffice import UserModel
from oemfoe_todos_app.models import TodoList, TodoItem


class TodoListModel(BackOfficeModel):
    model = TodoList
    parent_field = 'owner'
    siblings = 'user'

    tabs = (
        BackOfficeTab('todolist_details', 'Details'),
    )

    def get_title(self, instance):
        return instance.name

    def serialize(self, instance):
        return {'name': instance.name,
                'items': [item.__dict__ for item in instance.todoitem_set.all()]}

    def add_item(self, request):
        item = request.action_params.get('item')
        todo_list = get_object_or_404(TodoList, pk=request.action_params.get('todo_list'))
        if item:
            TodoItem.objects.create(todo_list=todo_list, name=item)
        return {'items': [item.__dict__ for item in todo_list.todoitem_set.all()]}

    def remove_item(self, request):
        todo_item = get_object_or_404(TodoItem, pk=request.action_params.get('todo_item'))
        todo_item.delete()


class TodoItemModel(BackOfficeModel):
    model = TodoItem
    parent_field = 'todo_list'

    tabs = (
        BackOfficeTab('todoitem_details', 'Details'),
    )

    def get_title(self, instance):
        return instance.name

    def serialize(self, instance):
        return {'name': instance.name,
                'todo_list_id': instance.todo_list.pk}

    def edit(self, request):
        todo_item = get_object_or_404(TodoItem, pk=request.action_params.get('todo_item'))
        todo_item.name = request.action_params.get('name')
        todo_item.save()
        messages.success(request, 'Successfully edited todo item!')


UserModel.tabs += (
    BackOfficeTab('todolists', 'Todo lists'),
)


class TodoListsView(BackOfficeView):
    def get_extra_context(self, request):
        user_id = request.view_params.get('user_id')
        return {'todo_lists': TodoList.objects.filter(owner=user_id)}

    def add_list(self, request):
        user_id = request.view_params.get('user_id')
        name = request.action_params.get('name')
        TodoList.objects.create(owner_id=user_id, name=name)
        messages.success(request, 'Successfully added todo list.')

    def remove_list(self, request):
        todo_list = TodoList.objects.get(pk=request.action_params.get('id'))
        todo_list.delete()
        messages.success(request, 'Successfully deleted todo list.')


from django.conf.urls import patterns, url
from advanced_reports.views import *

urlpatterns = patterns('',
    url(r'^(?P<slug>[^/]+)/$', list, name='advanced_reports_list'),
    
    url(r'^(?P<slug>[^/]+)/form/(?P<method>[^/]+)/(?P<object_id>[^/]+)/$', ajax_form, name='advanced_reports_form'),
    url(r'^(?P<slug>[^/]+)/form/(?P<method>[^/]+)/(?P<object_id>[^/]+)/(?P<param>[^/]+)/$', ajax_form, name='advanced_reports_form'),
    
    url(r'^(?P<slug>[^/]+)/action/(?P<method>[^/]+)/(?P<object_id>[^/]+)/$', action, name='advanced_reports_action'),
    url(r'^(?P<slug>[^/]+)/action/(?P<method>[^/]+)/(?P<object_id>[^/]+)/(?P<param>[^/]+)/$', action, name='advanced_reports_action'),
    
    url(r'^(?P<slug>[^/]+)/ajax/(?P<method>[^/]+)/(?P<object_id>[^/]+)/$', ajax, name='advanced_reports_ajax'),
    url(r'^(?P<slug>[^/]+)/ajax/(?P<method>[^/]+)/(?P<object_id>[^/]+)/(?P<param>[^/]+)/$', ajax, name='advanced_reports_ajax'),
    
    url(r'^(?P<slug>[^/]+)/count/$', count, name='advanced_reports_count'),

    url(r'^api/(?P<slug>[^/]+)/$', api_list, name='advanced_reports_api_list'),
    url(r'^api/(?P<slug>[^/]+)/action/(?P<method>[^/]+)/(?P<object_id>[^/]+)/$', api_action, name='advanced_reports_api_action'),
    url(r'^api/(?P<slug>[^/]+)/form/(?P<method>[^/]+)/(?P<object_id>[^/]+)/$', api_form,
        name='advanced_reports_api_form'),
    url(r'^api/(?P<slug>[^/]+)/form/(?P<method>[^/]+)/$', api_form,
        name='advanced_reports_api_form'),
)



from django.conf.urls import patterns, include, url

# Uncomment the next two lines to enable the admin:
from django.contrib import admin
from django.views.generic.base import RedirectView

from advanced_reports.backoffice.examples.backoffice import test_backoffice
from advreport_test_project.backoffice_definitions import todos_backoffice

admin.autodiscover()

urlpatterns = patterns('',
    url(r'^$', RedirectView.as_view(url='/todos-backoffice/'), name='home'),
    url(r'^admin/', include(admin.site.urls)),
    url(r'^test-backoffice/', include(test_backoffice.urls)),
    url(r'^todos-backoffice/', include(todos_backoffice.urls)),
    url(r'^reports/', include('advanced_reports.urls')),
)



from functools import wraps
from advanced_reports import get_report_or_404

try:
    from django.utils.timezone import now
except:
    import datetime
    now = datetime.datetime.now


def conditional_delegation(condition_func):
    """
    Delegate a view depending on a certain condition.
    """
    def decorator(view_func):
        @wraps(view_func)
        def delegated_view(request, *args, **kwargs):
            # Only do the actual delegation if the django-delegation library is installed.
            try:
                from django_delegation.decorators import delegate
                from django_delegation.utils import SimpleHTTPRequest
            except ImportError:
                return view_func(request, *args, **kwargs)

            if not isinstance(request, SimpleHTTPRequest) and condition_func(request):
                return delegate(view_func)(request, *args, **kwargs)
            return view_func(request, *args, **kwargs)
        return delegated_view
    return decorator


def report_view(view_func):
    """
    Encapsulates a view which deals with a report and applies a custom decorator
    defined in the report, if it has ``decorate_views = True``.
    """
    @wraps(view_func)
    def decorated(request, slug, *args, **kwargs):
        report = get_report_or_404(slug)
        report.set_request(request)
        inner = view_func
        if report.decorate_views:
            inner = report.get_decorator()(inner)
        return inner(request, report, *args, **kwargs)
    return decorated
